#include "sdlskk_internal.h"
#include <stdlib.h>
#include <string.h>

int SDLSKK_Str_len( SDLSKK_Char* str )
{
  int i;
  for( i=0; str[i] != 0; ++i )
    ;
  return i;
}

#ifdef USE_UTF8
char* SDLSKK_Str_to_cstr( SDLSKK_Char* str, unsigned char* cstr, size_t size )
{
  int str_pos, cstr_pos;

  for( str_pos=0, cstr_pos=0; str[str_pos] != 0; ++str_pos ){
    if( str[str_pos] <= 0x7F ){
      if( cstr_pos + 1 >= size )
	return NULL;
      cstr[cstr_pos] = str[str_pos];
      ++cstr_pos;
    }else if( str[str_pos] >= 0x80 && str[str_pos] <= 0x7FF ){
      if( cstr_pos + 2 >= size )
	return NULL;
      cstr[cstr_pos] = ((str[str_pos]>>6)&0x1F)|0xC0;
      cstr[cstr_pos+1] = (str[str_pos]&0x3F)|0x80;
      cstr_pos += 2;
    }else if( str[str_pos] >= 0x800 && str[str_pos] <= 0xFFFF ){
      if( cstr_pos + 3 >= size )
	return NULL;
      cstr[cstr_pos] = ((str[str_pos] >> 12) & 0x0F) | 0xE0;
      cstr[cstr_pos+1] = ((str[str_pos] >> 6) & 0x3F) | 0x80;
      cstr[cstr_pos+2] = (str[str_pos] & 0x3F) | 0x80;
      cstr_pos += 3;
    }else if( str[str_pos] >= 0x10000 && str[str_pos] <= 0x1FFFFF ){
      if( cstr_pos + 4 >= size )
	return NULL;
      cstr[cstr_pos] = ((str[str_pos] >> 18) & 0x07) | 0xF0;
      cstr[cstr_pos+1] = ((str[str_pos] >> 12) & 0x3F) | 0x80;
      cstr[cstr_pos+2] = ((str[str_pos] >> 6) & 0x3F) | 0x80;
      cstr[cstr_pos+3] = (str[str_pos] & 0x3F) | 0x80;
      cstr_pos += 4;
    }else if( str[str_pos] >= 0x200000 && str[str_pos] <= 0x3FFFFFF ){
      if( cstr_pos + 5 >= size )
	return NULL;
      cstr[cstr_pos] = ((str[str_pos] >> 24) & 0x03) | 0xF8;
      cstr[cstr_pos+1] = ((str[str_pos] >> 18) & 0x3F) | 0x80;
      cstr[cstr_pos+2] = ((str[str_pos] >> 12) & 0x3F) | 0x80;
      cstr[cstr_pos+3] = ((str[str_pos] >> 6) & 0x3F) | 0x80;
      cstr[cstr_pos+4] = (str[str_pos] & 0x3F) | 0x80;
      cstr_pos += 5;
    }else if( str[str_pos] >= 0x4000000 && str[str_pos] <= 0x7FFFFFFF ){
      if( cstr_pos + 6 >= size )
	return NULL;
      cstr[cstr_pos] = ((str[str_pos] >> 30) & 0x01) | 0xFC;
      cstr[cstr_pos+1] = ((str[str_pos] >> 24) & 0x3F) | 0x80;
      cstr[cstr_pos+2] = ((str[str_pos] >> 18) & 0x3F) | 0x80;
      cstr[cstr_pos+3] = ((str[str_pos] >> 12) & 0x3F) | 0x80;
      cstr[cstr_pos+4] = ((str[str_pos] >> 6) & 0x3F) | 0x80;
      cstr[cstr_pos+5] = (str[str_pos] & 0x3F) | 0x80;
      cstr_pos += 6;
    }else{
      return NULL;
    }    
  }
  cstr[cstr_pos]='\0';
  return cstr;
}
#else
char* SDLSKK_Str_to_cstr( SDLSKK_Char* str, unsigned char* cstr, size_t size )
{
  int str_pos, cstr_pos, i;
  unsigned char c;
  for( str_pos = cstr_pos = 0; str[str_pos] ; ++str_pos ){
    for( i=3; i>=0; --i ){
      c = ( str[str_pos] & ( ((unsigned int)0xFF) << i*8 ) ) >> i*8;
      if( c != 0 ){
	cstr[cstr_pos] = c;
	++cstr_pos;
	if( cstr_pos >= size )
	  return NULL;
      }
    }
  }
  cstr[cstr_pos] = '\0';
  return cstr;    
}
#endif

int SDLSKK_str_equal( SDLSKK_Char *str1, SDLSKK_Char *str2 )
{
  int i;

  if( (str1 == NULL)||(str2 == NULL) )
    return 0;
  
  for( i=0; str1[i] != 0; ++i ){
    if( str1[i] != str2[i] )
      return 0;
  }
  if( str2[i] == 0 )
    return 1;
  else
    return 0;
}

SDLSKK_Char* SDLSKK_Str_new( SDLSKK_Char * str )
{
  int len;
  int i;
  SDLSKK_Char* result;
  for( i=0; str[i] != 0; i++ )
    ;
  len = i+1;
  result = SDLSKK_malloc( len * sizeof(SDLSKK_Char) );
  memcpy( result, str, len * sizeof(SDLSKK_Char) );
  return result;
}

SDLSKK_Char* SDLSKK_Str_new2( char* str )
{
  SDLSKK_Char* buf;
  SDLSKK_Char* new_str;
  int str_pos, buf_pos;
  
  buf = SDLSKK_malloc( (strlen(str)+1)*sizeof(SDLSKK_Char) );
  for( str_pos = 0, buf_pos = 0; str[str_pos] != '\0'; ++buf_pos ){
    buf[buf_pos] = SDLSKK_get_sc( str, &str_pos );
  }
  buf[buf_pos] = 0;
  new_str = SDLSKK_Str_new( buf );
  free( buf );
  return new_str;
}

SDLSKK_Char* SDLSKK_Str_insert_Char( SDLSKK_Char* str, int pos, SDLSKK_Char c, int max_buf )
{
  int len;
  len = SDLSKK_Str_len(str);
  
  if(  len >= max_buf - 1 ){
    return NULL;
  }
  if( pos < 0 || pos > len )
    return NULL;
  memmove( &str[pos+1], &str[pos], (len-pos+1)*sizeof(SDLSKK_Char) );
  str[pos] = c;
  return str;
}

SDLSKK_Char* SDLSKK_Str_insert_Str( SDLSKK_Char* str, int pos, SDLSKK_Char *istr, int max_buf )
{
  int str_len, istr_len;
    
  str_len = SDLSKK_Str_len( str );
  istr_len = SDLSKK_Str_len( istr );
  if( str_len + istr_len + 1 >= max_buf )
    return NULL;
  if( pos < 0 || pos > str_len )
    return NULL;
  memmove( &str[pos+istr_len], &str[pos], (str_len-pos+1)*sizeof(SDLSKK_Char)  );
  memmove( &str[pos], istr, istr_len*sizeof(SDLSKK_Char) );
  return str;
}

SDLSKK_Char* SDLSKK_Str_delete_Char( SDLSKK_Char* str, int pos )
{
  if( pos < 0 || pos >= SDLSKK_Str_len( str ) )
    return NULL;
  memmove( &str[pos], &str[pos+1], (SDLSKK_Str_len(str)-pos+1)*sizeof(SDLSKK_Char) );
  return str;
}

void SDLSKK_Str_clear( SDLSKK_Char* str )
{
  str[0] = 0;
}

#ifdef USE_UTF8
SDLSKK_Char SDLSKK_get_sc( unsigned char* line, int* pos )
{
  SDLSKK_Char result;
  int len;
  int i;

  if( ( line[*pos] & 0x80 ) == 0 ){
    ++*pos;
    return line[ *pos-1 ];
  }else{
    for( i=0; ((0x80>>i)&line[*pos])!=0; i++ )
      ;
    len = i;
  }
  if( len>6 )
    return 0xFFFFFFFF;
  result = (0xFFU>>len) & line[*pos];
  for( i=1; i<len; ++i ){
    result = ( result << 6 )|( line[*pos+i] & 0x3F );
  }
  *pos += len;
  return result;
}
#else
SDLSKK_Char SDLSKK_get_sc( unsigned char* line, int* pos )
{
  SDLSKK_Char result;
  
  /* assume EUC_JP */
  if( line[ *pos ] <= 0x7E ){
    result = line[ *pos ];
    ++*pos;
    return result;
  }
  if( line[ *pos ] >= 0xA0 && line[ *pos ] <= 0xFF ){
    result = (line[ *pos ]<<8) + line[ *pos + 1 ];
    *pos += 2;
    return result;
  }
  if( line[ *pos ] == 0x8E ){
    result = (line[ *pos ]<<8) + line[ *pos + 1 ];
    *pos += 2;
    return result;
  }
  if( line[ *pos ] == 0x8F ){
    result = (line[ *pos ]<<16) + (line[ *pos + 1 ]<<8) + line[ *pos + 2 ];
    *pos += 3;
    return result;
  }
  return 0xFFFFFFFF;
}
#endif

SDLSKK_Char* SDLSKK_cstr_tokenize( char* line, char delimiter, int* pos )
{
  int i;
  SDLSKK_Char* str = SDLSKK_malloc( (strlen(line)+1)*sizeof(SDLSKK_Char) );
  SDLSKK_Char* new_str;
  
  i = -1;
  do {
    ++i;
    str[i] = SDLSKK_get_sc( line, pos );
  }while( !( str[i] == 0 || str[i] == delimiter || str[i] == '\n') );

  str[i] = 0;
  new_str = SDLSKK_Str_new( str );
  free( str );
  return new_str;
}

/* like String#chop! on Ruby */
void SDLSKK_Str_chop( SDLSKK_Char* str )
{
  if( SDLSKK_Str_len( str ) == 0 )
    return ;
  str[ SDLSKK_Str_len( str ) -1 ] = 0;
}

/* SDLSKK_XStr */
SDLSKK_XStr* SDLSKK_XStr_new1( SDLSKK_Char* str )
{
  SDLSKK_XStr* new_str = SDLSKK_malloc( sizeof(SDLSKK_XStr) );
  
  new_str->len = SDLSKK_Str_len( str );
  new_str->max = new_str->len + 1;
  new_str->buf = SDLSKK_Str_new( str );
  return new_str;
}

#define SDLSKK_MAX(x,y) ( ((x)>(y))?(x):(y) )
SDLSKK_XStr* SDLSKK_XStr_new2( SDLSKK_Char* str, int maxlen )
{
  SDLSKK_XStr* new_str = SDLSKK_malloc( sizeof(SDLSKK_XStr) );

  new_str->len = SDLSKK_Str_len( str );
  new_str->max = SDLSKK_MAX( new_str->len + 1, maxlen );
  new_str->buf = SDLSKK_malloc( new_str->max*sizeof(SDLSKK_Char) );
  memcpy( new_str->buf, str, (new_str->len + 1)*sizeof(SDLSKK_Char) );
  return new_str;
}

SDLSKK_XStr* SDLSKK_XStr_new3( char* str )
{
  SDLSKK_XStr* new_str = SDLSKK_malloc( sizeof(SDLSKK_XStr) );

  new_str->buf = SDLSKK_Str_new2( str );  
  new_str->len = SDLSKK_Str_len( new_str->buf );
  new_str->max = new_str->len + 1;
  
  return new_str;
}

SDLSKK_XStr* SDLSKK_XStr_new4( char* str, int maxlen )
{
  SDLSKK_XStr* new_str;
  SDLSKK_Char* tmp;
  
  tmp = SDLSKK_Str_new2( str );
  new_str = SDLSKK_XStr_new2( tmp, maxlen );
  free( tmp );
  return new_str;
}

SDLSKK_XStr* SDLSKK_XStr_new5( SDLSKK_XStr* str )
{
  return SDLSKK_XStr_new2( str->buf, str->max );
}

static void XStr_realloc( SDLSKK_XStr* str, size_t size )
{
  int newstr_max_len;
  
  if( size + 1 >= str->max  ){
    newstr_max_len = ( size + 1 )*2;
    str->buf = SDLSKK_realloc( str->buf, newstr_max_len*sizeof(SDLSKK_Char) );
    str->max = newstr_max_len;
  }
}
void SDLSKK_XStr_delete( SDLSKK_XStr* str )
{
  free( str->buf );
  free( str );
}

void SDLSKK_XStr_insert_Char( SDLSKK_XStr* str, int pos, SDLSKK_Char c )
{
  XStr_realloc( str, str->len + 1 );
  if( SDLSKK_Str_insert_Char( str->buf, pos, c, str->max ) != NULL )
    ++(str->len);
  
}

void SDLSKK_XStr_insert_Str( SDLSKK_XStr* str, int pos, SDLSKK_Char *istr )
{
  XStr_realloc( str, str->len + SDLSKK_Str_len(istr) );
  if( SDLSKK_Str_insert_Str( str->buf, pos, istr, str->max ) != NULL )
    str->len += SDLSKK_Str_len(istr);
}

void SDLSKK_XStr_delete_Char( SDLSKK_XStr* str, int pos )
{
  if( SDLSKK_Str_delete_Char( str->buf, pos ) )
    str->len -= 1;
}

void SDLSKK_XStr_clear( SDLSKK_XStr* str )
{
  str->buf[0] = 0;
  str->len = 0;
}
void SDLSKK_XStr_concat_Str( SDLSKK_XStr* str1, SDLSKK_Char* str2 )
{
  SDLSKK_XStr_insert_Str( str1, str1->len, str2 );  
}

void SDLSKK_XStr_copy_Str( SDLSKK_XStr* str1, SDLSKK_Char* str2 )
{
  SDLSKK_XStr_clear( str1 );
  SDLSKK_XStr_insert_Str( str1, 0, str2 );
}

void SDLSKK_XStr_slice_Str( SDLSKK_XStr* dest, SDLSKK_Char* src, int start, int end )
{
  int len = end - start + 1;
  
  if( !( 0 <= start  && start <= end && end <= SDLSKK_Str_len( src ) ) )
    return;
  
  XStr_realloc( dest, len );
  memmove( dest->buf, &src[start], len * sizeof(SDLSKK_Char) );
  dest->buf[len] = 0;
  dest->len = len;
}

#include "jisx0208_latin_table.tbl"
SDLSKK_Char SDLSKK_latin_to_jisx0208_Char( SDLSKK_Char ch )
{
  return jisx0208_latin_table[ch];
}

#ifdef USE_UTF8
SDLSKK_Char SDLSKK_hiragana_to_katakana( SDLSKK_Char ch )
{
  if( ch >= 0x3041 && ch <= 0x3096 )
    return ch + 0x60;
  return ch;
}
SDLSKK_Char SDLSKK_katakana_to_hiragana( SDLSKK_Char ch )
{
  if( ch >= 0x30A1 && ch <= 0x30F6 )
    return ch - 0x60;
  return ch;
}
#else
SDLSKK_Char SDLSKK_hiragana_to_katakana( SDLSKK_Char ch )
{
  if( ch >= 0xA4A1 && ch <= 0xA4F3 )
    return ch + 0x0100;
  return ch;
}
SDLSKK_Char SDLSKK_katakana_to_hiragana( SDLSKK_Char ch )
{
  if( ch >= 0xA5A1 && ch <= 0xA5F3 )
    return ch - 0x0100;
  return ch;
}
#endif
SDLSKK_Char* SDLSKK_Str_hiragana_to_katakana( SDLSKK_Char* str )
{
  SDLSKK_Char* new_str;
  int i;
  
  new_str = SDLSKK_Str_new( str );
  for( i=0; new_str[i] != 0 ; i++ )
    new_str[i] = SDLSKK_hiragana_to_katakana( new_str[i] );
  
  return new_str;
}
SDLSKK_Char* SDLSKK_Str_katakana_to_hiragana( SDLSKK_Char* str )
{
  SDLSKK_Char* new_str;
  int i;
  
  new_str = SDLSKK_Str_new( str );
  for( i=0; new_str[i] != 0 ; i++ )
    new_str[i] = SDLSKK_katakana_to_hiragana( new_str[i] );
  
  return new_str;
}


/* for debug */
void SDLSKK_Char_to_char( SDLSKK_Char c, char *result )
{
  int j,k;
  unsigned char b;
  
  for( j=3, k=0; j>=0; --j ){
    b = ( c & ( ((unsigned int)0xFF) << j*8 ) ) >> j*8;
    if( b != 0 ){
      result[k] = b;
      ++k;
    }
  }
  result[k] = '\0';  
}

void print_Str( SDLSKK_Char* str )
{
  char buf[1000];
  SDLSKK_Str_to_cstr( str, buf, sizeof(buf) );
  printf( "%s", buf );
}
