#include "sdlskk_internal.h"
#include <stdlib.h>

static unsigned int hash_value(SDLSKK_Char *key)
{
  int i;
  unsigned int hash=0;
  
  for( i=0; key[i] != 0; i++ ){
    hash = hash*65599 + key[i];
  }
  hash = hash + (hash>>5);
  return hash;
}

SDLSKK_Map* SDLSKK_Map_new( int table_size )
{
  SDLSKK_Map* map = SDLSKK_malloc( sizeof(SDLSKK_Map) );
  int i;
  map->table = SDLSKK_malloc( table_size * sizeof(SDLSKK_HashElement*) );
  map->table_size = table_size;
  for( i=0; i < table_size; ++i )
    map->table[i] = NULL;
  return map;
}

void SDLSKK_Map_set( SDLSKK_Map* map, SDLSKK_Char* key, SDLSKK_StrList *vals )
{
  SDLSKK_HashElement* element;
  SDLSKK_HashElement* new_element;
  
  if( map == NULL )return;    

  for( element = map->table[ hash_value(key) % map->table_size ];
       element != NULL;       
       element = element->next
        ){
    if( SDLSKK_str_equal( element->key, key ) ){
      SDLSKK_StrList_delete( element->vals );
      element->vals = vals;
      return;
    }
  }
    
  new_element = SDLSKK_malloc(sizeof(SDLSKK_HashElement));
  new_element->next = map->table[ hash_value(key) % map->table_size ];
  new_element->key = key;
  new_element->vals = vals;
  map->table[ hash_value(key) % map->table_size ] = new_element;
  return ;
}
      
SDLSKK_StrList* SDLSKK_Map_get( SDLSKK_Map* map, SDLSKK_Char* key )
{
  SDLSKK_HashElement* element;
  
  for( element = map->table[ hash_value(key) % map->table_size ];
       element != NULL;
       element = element->next
       ){
    if( SDLSKK_str_equal( element->key, key ) ){
      return element->vals;
    }
  }
  return NULL;
}
      
void SDLSKK_Map_delete( SDLSKK_Map* map )
{
  int i;
  SDLSKK_HashElement *element;
  SDLSKK_HashElement *next_element;
  
  for( i=0; i < map->table_size; i++ ){
    for( element = map->table[i]; element != NULL; ){
      SDLSKK_StrList_delete( element->vals );
      next_element = element->next;
      free( element );
      element = next_element;
    }
  }
  return;
}

void SDLSKK_Map_delete_all( SDLSKK_Map* map )
{
  int i;
  SDLSKK_HashElement *element;
  SDLSKK_HashElement *next_element;
  
  for( i=0; i < map->table_size; i++ ){
    for( element = map->table[i]; element != NULL;  ){
      SDLSKK_StrList_delete_all( element->vals );
      free( element->key );
      next_element = element->next;
      free( element );
      element = next_element;
    }
  }
  return;
}
