#include "sdlskk_internal.h"
#include "testutility.h"

static void Context_input_chars( SDLSKK_Context* context, const char* input )
{
  int i;
  for( i=0; input[i] != '\0'; ++i )
    SDLSKK_Context_input_key( context, input[i] );
}
/* memory leakƤޤɤƥȤ̵ */

void test_SDSKK_Str_len( void* data )
{
  NEQUAL( " length of \"\"", 0, SDLSKK_Str_len( SDLSKK_Str_new2( "" ) ) );
  NEQUAL( " length of \"XYZ\"", 3,
	  SDLSKK_Str_len( SDLSKK_Str_new2( "XYZ" ) ) );
  NEQUAL( " length of \"ۤ\"", 2,
	  SDLSKK_Str_len( SDLSKK_Str_new2( "ۤ" ) ) );  
}

void test_SDLSKK_Str_insert_Char( void* data )
{
  SDLSKK_Char str1[100] = { 's', 'p', 'q', 'r', 0 };
  SDLSKK_Char* str2 =  SDLSKK_Str_new2( "spXqr" );
  SDLSKK_Char* str3 =  SDLSKK_Str_new2( "YspXqr" );
  SDLSKK_Str_insert_Char( str1, 2, 'X', 100 );
  SSSEQUAL( "same Str", str2, str1 );
  SDLSKK_Str_insert_Char( str1, 0, 'Y', 7 );
  SSSEQUAL( "enough buffer size", str3, str1 );
  SDLSKK_Str_insert_Char( str1, 0, 'Y', 7 );
  SSSEQUAL( "not enough buffer size", str3, str1 );
  free( str2 );
  free( str3 );
}

void test_SDLSKK_Str_insert_Str( void* data )
{
  SDLSKK_Char str1[100] = { 's', 'p', 'q', 'r', 0 };
  SDLSKK_Char* str2 =  SDLSKK_Str_new2( "spXYqr" );
  SDLSKK_Char* str3 =  SDLSKK_Str_new2( "YqZspXYqr" );
  SDLSKK_Char* str4 =  SDLSKK_Str_new2( "YqZspXYqrIIZZ" );
  
  SDLSKK_Str_insert_Str( str1, 2, SDLSKK_Str_new2("XY"), 100 );
  SSSEQUAL( "insert str", str2, str1 );
  
  SDLSKK_Str_insert_Str( str1, 0, SDLSKK_Str_new2("YqZ"), 100 );
  SSSEQUAL( "insert str head", str3, str1 );
  
  SDLSKK_Str_insert_Str( str1, 9, SDLSKK_Str_new2("IIZZ"), 100 );
  SSSEQUAL( "insert str tail", str4, str1 );
  
  SDLSKK_Str_insert_Str( str1, 0, SDLSKK_Str_new2("IIZZ"), 18 );
  SSSEQUAL( "insert str length check", str4, str1 );
}

void test_SDLSKK_Str_to_cstr( void* data )
{
  char buf[100];
  
  SDLSKK_Str_to_cstr( SDLSKK_Str_new2( "" ), buf, 100 );
  SEQUAL( "cstr -> Str -> cstr \"\"", "", buf );
  
  SDLSKK_Str_to_cstr( SDLSKK_Str_new2( "ۤ" ), buf, 100 );
  SEQUAL( "cstr -> Str -> cstr \"ۤ\"", "ۤ", buf );
  
  PEQUAL( "SDLSKK_Str_to_cstr buffer overflow check",
	  NULL, SDLSKK_Str_to_cstr( SDLSKK_Str_new2( "ۤ" ), buf, 4 ) );
}
	  
/* SKK-JISYO.SȤ */  
void test_SDLSKK_DictItem( void* data )
{
  SDLSKK_Dictionary* dict;
  SDLSKK_DictionaryItem* item;
  SDLSKK_Char* key;
  
  dict = SDLSKK_Dict_new();
  SDLSKK_Dict_load( dict, "test_jisyo", 0 );
  key = SDLSKK_Str_new2( "" );
  item = SDLSKK_Dict_get( dict, key );

  SSSEQUAL( "get first kanji", SDLSKK_Str_new2(""),
	    SDLSKK_DictItem_get_current( item ) );
  SDLSKK_DictItem_next( item );
  SDLSKK_DictItem_next( item );
  SSSEQUAL( "get third kanji", SDLSKK_Str_new2("ȱ"),
	    SDLSKK_DictItem_get_current( item ) );
  SDLSKK_DictItem_next( item );
  SDLSKK_DictItem_next( item );
  SSSEQUAL( "get last kanji, 'next' can't go over the last item",
	    SDLSKK_Str_new2(""),
	    SDLSKK_DictItem_get_current( item ) );
  GASSERT( "last check", SDLSKK_DictItem_is_end( item ) );
  
  SDLSKK_DictItem_reset( item );
  SSSEQUAL( "get first kanji again, check reset", SDLSKK_Str_new2(""),
	    SDLSKK_DictItem_get_current( item ) );
  GASSERT( "first check", SDLSKK_DictItem_is_begin( item ) );
  
  SDLSKK_DictItem_delete( item );
  SDLSKK_Dict_delete_all( dict );
}

void test_SDLSKK_RomKanaRuleTable( void* data )
{
  SDLSKK_RomKanaRuleTable* rule_table;
  SDLSKK_RomKanaElement* candidate;
  
  rule_table = SDLSKK_RomKanaRuleTable_new( "test_rule_table" );
  
  NEQUAL( "RomKanaRuleTable \"pa\"", SDLSKK_RomKana_OneCandidate,
	  SDLSKK_RomKanaRuleTable_get( rule_table,
				       SDLSKK_Str_new2("pa"),
				       &candidate ) );
  SSSEQUAL( "RomKanaRuleTable \"pa\"", SDLSKK_Str_new2(""),
	    candidate->hiragana );
  
  NEQUAL( "RomKanaRuleTable \"py\"", SDLSKK_RomKana_SomeCandidates,
	  SDLSKK_RomKanaRuleTable_get( rule_table,
				       SDLSKK_Str_new2("py"),
				       &candidate ) );
  PEQUAL( "RomKanaRuleTable \"p\"", NULL, candidate );
  
  NEQUAL( "RomKanaRuleTable \"n\"", SDLSKK_RomKana_UnresolvedCandidate,
	  SDLSKK_RomKanaRuleTable_get( rule_table,
				       SDLSKK_Str_new2("n"),
				       &candidate ) );
  SSSEQUAL( "RomKanaRuleTable \"n\"", SDLSKK_Str_new2(""),
	    candidate->hiragana );

  NEQUAL( "RomKanaRuleTable \"nh\"", SDLSKK_RomKana_NoCandidate,
	  SDLSKK_RomKanaRuleTable_get( rule_table,
				       SDLSKK_Str_new2("nh"),
				       &candidate ) );
  PEQUAL( "RomKanaRuleTable \"nh\"", NULL, candidate );
  SDLSKK_RomKanaRuleTable_delete( rule_table );
}

void test_SDLSKK_Map( void *data )
{
  SDLSKK_Map* map; 
  SDLSKK_StrList* list;
  
  map = SDLSKK_Map_new( 1024 );
  list = SDLSKK_StrList_new();
  SDLSKK_Map_set( map, SDLSKK_Str_new2( "fuga" ), list );
  PEQUAL( "SDLSKK_Map get \"fuga\"", list,
	  SDLSKK_Map_get( map, SDLSKK_Str_new2("fuga") ) );
  PEQUAL( "SDLSKK_Map get \"hoge\"", NULL,
	  SDLSKK_Map_get( map, SDLSKK_Str_new2("hoge") ) );
  SDLSKK_Map_delete_all( map );

  map = SDLSKK_Map_new( 1024 );
  list = SDLSKK_StrList_new();
  SDLSKK_Map_set( map, SDLSKK_Str_new2( "fuga" ), list );
  SDLSKK_Map_delete( map );
}

void test_SDLSKK_Context__input_hiragana( void* data )
{
  SDLSKK_Context* context;
  SDLSKK_RomKanaRuleTable* table;
  table = SDLSKK_RomKanaRuleTable_new( "test_rule_table" );
  context = SDLSKK_Context_new( NULL, table );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SDLSKK_Str_new2(""), context->str->buf );
  SDLSKK_Context_input_key( context, 'p' );
  SSSEQUAL( "", SDLSKK_Str_new2(""), context->str->buf );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SDLSKK_Str_new2(""), context->str->buf );
  SDLSKK_Context_input_key( context, 'n' );
  SSSEQUAL( "", SDLSKK_Str_new2(""), context->str->buf );
  SDLSKK_Context_input_key( context, 'p' );
  SSSEQUAL( "", SDLSKK_Str_new2("Ѥ"), context->str->buf );
  SDLSKK_Context_input_key( context, 'y' );
  SSSEQUAL( "", SDLSKK_Str_new2("Ѥ"), context->str->buf );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤ"), context->str->buf );  
  SDLSKK_Context_input_key( context, 'h' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤ"), context->str->buf );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤ"), context->str->buf );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤ"), context->str->buf );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤä"), context->str->buf );

  /* ϤΥƥ */
  SDLSKK_Context_input_key( context, '?' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤä"), context->str->buf );
  SDLSKK_Context_input_key( context, '[' );
  SSSEQUAL( "", SDLSKK_Str_new2("ѤԤä"), context->str->buf );

  SDLSKK_Context_delete( context );
}
  
void test_SDLSKK_XStr( void* data )
{
  SDLSKK_XStr* str;
  str = SDLSKK_XStr_new3( "ۤ" );
  
  SDLSKK_XStr_insert_Char( str, 1, 'h' );
  SSSEQUAL( "", SDLSKK_Str_new2( "h" ), str->buf );
  NEQUAL( "", 3, str->len );  
  GASSERT( "", str->max > 3 );
  
  SDLSKK_XStr_insert_Str( str, 0, SDLSKK_Str_new2("դ") );
  SSSEQUAL( "", SDLSKK_Str_new2( "դh" ), str->buf );
  NEQUAL( "", 5, str->len );
  SDLSKK_XStr_insert_Str( str, -1, SDLSKK_Str_new2("դۤ") );
  SSSEQUAL( "", SDLSKK_Str_new2( "դh" ), str->buf );
  NEQUAL( "", 5, str->len );
  SDLSKK_XStr_insert_Str( str, 6, SDLSKK_Str_new2("դۤ") );
  SSSEQUAL( "", SDLSKK_Str_new2( "դh" ), str->buf );
  NEQUAL( "", 5, str->len );
  
  SDLSKK_XStr_delete_Char( str, 8 );
  SSSEQUAL( "", SDLSKK_Str_new2( "դh" ), str->buf );
  NEQUAL( "", 5, str->len );
  SDLSKK_XStr_delete_Char( str, 3 );
  SSSEQUAL( "", SDLSKK_Str_new2( "դۤ" ), str->buf );
  NEQUAL( "", 4, str->len );
  SDLSKK_XStr_delete_Char( str, 0 );
  SSSEQUAL( "", SDLSKK_Str_new2( "ۤ" ), str->buf );
  NEQUAL( "", 3, str->len );

  SDLSKK_XStr_concat_Str( str, SDLSKK_Str_new2( "" ) );
  SSSEQUAL( "", SDLSKK_Str_new2( "ۤ" ), str->buf );
  NEQUAL( "", 5, str->len );
  
  SDLSKK_XStr_clear( str );
  SSSEQUAL( "", SDLSKK_Str_new2( "" ), str->buf );
  NEQUAL( "", 0, str->len );

  SDLSKK_XStr_copy_Str( str, SDLSKK_Str_new2( "" ) );
  SSSEQUAL( "", SDLSKK_Str_new2( "" ), str->buf );
  NEQUAL( "", 2, str->len );

  SDLSKK_XStr_slice_Str( str, SDLSKK_Str_new2( "ƤȤ" ), 2, 3 );
  SSSEQUAL0( SDLSKK_Str_new2( "Ȥ" ), str->buf );
  NEQUAL0( 2, str->len );

  SDLSKK_XStr_slice_Str( str, SDLSKK_Str_new2( "ƤȤ" ), 2, 2 );
  SSSEQUAL0( SDLSKK_Str_new2( "" ), str->buf );
  NEQUAL0( 1, str->len );
}

static SDLSKK_Char* SS( char* s )
{
  return SDLSKK_Str_new2( s );
}
void test_SDLSKK_Context_get_display_Str__input_alpha_hiragana( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'h' );
  SSSEQUAL( "", SS("h"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'p' );
  SSSEQUAL( "", SS("p"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'y' );
  SSSEQUAL( "", SS("py"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'o' );
  SSSEQUAL( "", SS("ؤԤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SS("ؤԤt"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SSSEQUAL( "", SS("ؤԤk"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'y' );
  SSSEQUAL( "", SS("ؤԤky"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'l' );
  SSSEQUAL( "", SS("ؤԤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'x' );
  SSSEQUAL( "", SS("ؤԤx"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'p' );
  SSSEQUAL( "", SS("ؤԤxp"), SDLSKK_Context_get_display_Str( context ) );
  
  SDLSKK_Context_input_key( context, CTRL('j') );
  /* test input backspace */
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("ؤԤxp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("ؤԤxp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'h' );
  SSSEQUAL( "", SS("ؤԤxph"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("ؤԤxp"), SDLSKK_Context_get_display_Str( context ) );
  /* test input backward and forward */
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, 'm' );
  SSSEQUAL( "", SS("ؤԤxmp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("ؤԤxp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, CTRL('f') );
  SDLSKK_Context_input_key( context, CTRL('f') );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SSSEQUAL( "", SS("ؤԤ礱kxp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SS("ؤԤ礱txp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('f') );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );

  /* test delete */
  SDLSKK_Context_input_key( context, CTRL('d') );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('f') );
  SDLSKK_Context_input_key( context, CTRL('d') );
  SSSEQUAL( "", SS("ؤԤ礱xp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, 'h' );
  SSSEQUAL( "", SS("ؤԤ礱xhp"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('d') );
  SSSEQUAL( "", SS("ؤԤ礱x"), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_Context__input_kanji_okuri_nasi( void* data )
{
  SDLSKK_Context* context = data;
  
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'T' );
  SSSEQUAL( "", SS("t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'o' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SSSEQUAL( "", SS("k"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("Ȥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("Ȥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'A' );
  SSSEQUAL( "", SS("ע"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'm' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("ע"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("ע"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );

  /* ʤξΥƥ */
  SDLSKK_Context_input_key( context, 'H' );
  SDLSKK_Context_input_key( context, 'o' );
  SDLSKK_Context_input_key( context, 'g' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("װۤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("װۤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("װۤ"), SDLSKK_Context_get_display_Str( context ) );
  
  /* backspaceΥƥ */
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'w' );
  SSSEQUAL( "", SS("װw"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );

  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS("װ"), SDLSKK_Context_get_display_Str( context ) );

  /* l򲡤ȤΥƥ*/
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );

  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("Ƥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'l' );
  SSSEQUAL( "", SS("Ƥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("ƤTeki"), SDLSKK_Context_get_display_Str( context ) );
  
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'i' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("Ū"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'l' );
  SDLSKK_Context_input_key( context, 't' );
  SDLSKK_Context_input_key( context, 'E' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("ŪtEki"), SDLSKK_Context_get_display_Str( context ) );

  /* θɽƥ */
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("ŪtEki"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("ŪtEkiŹ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'x' );
  SSSEQUAL( "", SS("ŪtEki"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'x' );
  SSSEQUAL( "", SS("ŪtEki"), SDLSKK_Context_get_display_Str( context ) );

  /* ѴʸκǸ'n'ǡ֤פ򤢤綠Ȥƥ */
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, 'H' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("Ѵ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL( "", SS("Ѵ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'H' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, 's' );
  SDLSKK_Context_input_key( context, 'i' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("Ѵؤ󤷤"), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_Context__input_kanji_okuri_ari( void* data )
{
  SDLSKK_Context* context = data;
  
  SDLSKK_Context_input_key( context, 'M' );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'T' );
  SSSEQUAL( "", SS("*t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'u' );
  SSSEQUAL( "", SS("Ԥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL( "", SS("Ԥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  
  SDLSKK_Context_input_key( context, 'H' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'R' );
  SDLSKK_Context_input_key( context, 'u' );
  SSSEQUAL( "", SS("ޤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("פ"), SDLSKK_Context_get_display_Str( context ) );
  
  SDLSKK_Context_input_key( context, 'K' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'T' );
  SSSEQUAL( "", SS("פ뢦*t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SS("פ뢦*t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS("פ뢧ä"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'Y' );
  SDLSKK_Context_input_key( context, 'o' );
  SDLSKK_Context_input_key( context, 'I' );
  SSSEQUAL( "", SS("פ뵢äɤ"), SDLSKK_Context_get_display_Str( context ) );  
  
  /* backspaceΥƥ */
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );

  SDLSKK_Context_input_key( context, 'K' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'W' );
  SSSEQUAL( "", SS("*w"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, 'S' );
  SDLSKK_Context_input_key( context, 'h' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'b' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SS("*t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );

  /* ʤΥƥ */
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, 'H' );
  SDLSKK_Context_input_key( context, 'o' );
  SDLSKK_Context_input_key( context, 'G' );
  SSSEQUAL( "", SS("*g"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("¾"), SDLSKK_Context_get_display_Str( context ) );

}

void test_SDLSKK_Context__abbrev_mode( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, '/' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'l' );
  SDLSKK_Context_input_key( context, 'l' );
  SSSEQUAL( "", SS("all"),SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );

  /* backspaceΥƥ */
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, '/' );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS("a"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'h' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_Context_get_display_str( void* data )
{
  SDLSKK_Context* context = data;
  char buf[4098];
  
  SDLSKK_Context_get_display_str( context, buf, sizeof(buf) );
  SEQUAL0( "", buf );
  SDLSKK_Context_input_key( context, 'h' );
  SDLSKK_Context_get_display_str( context, buf, sizeof(buf) );
  SEQUAL0( "h", buf );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_get_display_str( context, buf, sizeof(buf) );
  SEQUAL0( "", buf );  
}

void test_SDLSKK_Context__input_kigou( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, '-' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, '7' );
  SSSEQUAL( "", SS("7"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'l' );
  SDLSKK_Context_input_key( context, '-' );
  SSSEQUAL( "", SS("7-"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, 'P' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, '-' );
  SSSEQUAL( "", SS("ڡ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'j' );
  SDLSKK_Context_input_key( context, 'i' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_Context__jisx0208_latin_mode( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, 'L' );
  SDLSKK_Context_input_key( context, 'p' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, '+' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );  
  SDLSKK_Context_input_key( context, CTRL('b') );
  SDLSKK_Context_input_key( context, 'K' );
  SSSEQUAL( "", SS("ˣ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("ˡ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, '/' );
  SSSEQUAL( "", SS("ˡ"), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_katakana_to_hiragana( void* data )
{
  NEQUAL0( SS("")[0], SDLSKK_katakana_to_hiragana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_katakana_to_hiragana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_katakana_to_hiragana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_katakana_to_hiragana( SS("")[0] ) );
}

void test_SDLSKK_hiragana_to_katakana( void* data )
{
  NEQUAL0( SS("")[0], SDLSKK_hiragana_to_katakana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_hiragana_to_katakana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_hiragana_to_katakana( SS("")[0] ) );
  NEQUAL0( SS("")[0], SDLSKK_hiragana_to_katakana( SS("")[0] ) );
}

void test_SDLSKK_Context__input_katakana( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, 'q' );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SSSEQUAL( "", SS("k"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'q' );
  SDLSKK_Context_input_key( context, 'a' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );

  /* Ϥƥ */
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, 'q' );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'k' );
  SSSEQUAL( "", SS("k"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'i' );
  SSSEQUAL( "", SS("ƥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("Ū"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS("Ũ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'N' );
  /* 겾̾δ */
  SSSEQUAL( "", SS("Ũn"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'o' );
  SDLSKK_Context_input_key( context, 'S' );
  SSSEQUAL( "", SS("Ũ*s"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS("Ũܤ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'r' );
  SDLSKK_Context_input_key( context, 'u' );
  SSSEQUAL( "", SS("Ũܤ"), SDLSKK_Context_get_display_Str( context ) );

  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );
  SDLSKK_Context_input_key( context, CTRL('h') );

  /* HENKAN_ONΤȤq򲡤Ѵ褦ȤƤʸʿ̾
     Ҳ̾ؤΤƥ */
  SDLSKK_Context_input_key( context, 'K' );
  SDLSKK_Context_input_key( context, 'o' );
  SDLSKK_Context_input_key( context, 'm' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'q' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'q' );
  SDLSKK_Context_input_key( context, 'E' );
  SSSEQUAL( "", SS("ᢦ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'q' );
  SSSEQUAL( "", SS("ᥨ"), SDLSKK_Context_get_display_Str( context ) );
  /* ˸nǤȤΥƥ */
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SSSEQUAL( "", SS("ᥨn"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'q' );
  SSSEQUAL( "", SS("ᥨƥ"), SDLSKK_Context_get_display_Str( context ) );
  /* ֤C-j򲡤ȤưΥƥ */
  SDLSKK_Context_input_key( context, 'q' );
  SDLSKK_Context_input_key( context, 'M' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL0( SS("ᥨƥ󢦥"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('j') );
  SSSEQUAL0( SS("ᥨƥ"), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_Context__input_cancel_key( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('g') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );

  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('g') );
  SSSEQUAL( "", SS("Ƥ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('g') );

  SDLSKK_Context_input_key( context, 'K' );
  SDLSKK_Context_input_key( context, 'a' );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 't' );
  SSSEQUAL( "", SS("*t"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, CTRL('g') );
  SSSEQUAL( "", SS(""), SDLSKK_Context_get_display_Str( context ) );
}

void test_SDLSKK_DictItem_commit( void* data )
{
  SDLSKK_Context* context = data;

  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("Ź"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("ŷ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, 'T' );
  SDLSKK_Context_input_key( context, 'e' );
  SDLSKK_Context_input_key( context, 'n' );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("ŷŷ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("ŷ"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("ŷŹ"), SDLSKK_Context_get_display_Str( context ) );
  
}

void test_SDLSKK_Dict_save_user_jisyo( void* data )
{
  SDLSKK_Context* context = data;
  SDLSKK_Dictionary* dict = context->dict;

  Context_input_chars( context, "Ten  Ten  WakaIl" );
  
  SDLSKK_Dict_save_user_dict( dict, "test_saved_user_jisyo0" );
  /* ǥ֤ϤȤshell scriptdiffƤӤƥƥȤ */
  /* ĤǤˤ shell script ǥ֤եrm */
}

void test_SDLSKK_Dict_load( void* data )
{
  SDLSKK_Context* context;
  SDLSKK_RomKanaRuleTable* table;
  SDLSKK_Dictionary* dict;
  
  dict = SDLSKK_Dict_new();
  SDLSKK_Dict_load( dict, "saved_user_jisyo0", 1 );
  SDLSKK_Dict_load( dict, "test_jisyo", 0 );
  
  table = SDLSKK_RomKanaRuleTable_new( "test_rule_table" );
  context = SDLSKK_Context_new( dict, table );
  
  Context_input_chars( context, "Ten " );
  SSSEQUAL0( SS(""), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("Ź"), SDLSKK_Context_get_display_Str( context ) );
  SDLSKK_Context_input_key( context, ' ' );
  SSSEQUAL0( SS("ŷ"), SDLSKK_Context_get_display_Str( context ) );

  SDLSKK_Context_delete( context );
  SDLSKK_RomKanaRuleTable_delete( table );
  SDLSKK_Dict_delete_all( dict );
}

void setup_SDLSKK_Context( void** data )
{
  SDLSKK_Context* context;
  SDLSKK_RomKanaRuleTable* table;
  SDLSKK_Dictionary* dict;
  
  dict = SDLSKK_Dict_new();
  SDLSKK_Dict_load( dict, "test_jisyo", 0 );
  table = SDLSKK_RomKanaRuleTable_new( "test_rule_table" );
  context = SDLSKK_Context_new( dict, table );
  *data = context;
}

void teardown_SDLSKK_Context( void** data )
{
  SDLSKK_Context* context;
  SDLSKK_RomKanaRuleTable* table;
  SDLSKK_Dictionary* dict;

  context = *data;
  table = context->rule_table;
  dict = context->dict;
  
  SDLSKK_Context_delete( context );
  SDLSKK_RomKanaRuleTable_delete( table );
  SDLSKK_Dict_delete_all( dict );
}

void test_SDLSKK_Str_hiragana_to_katakana( void* data )
{
  SSSEQUAL( "", SS("إ"), SDLSKK_Str_hiragana_to_katakana( SS("ؤ") ) );
  SSSEQUAL( "", SS(""), SDLSKK_Str_hiragana_to_katakana( SS("") ) );
}
void test_run()
{
  TEST_ENV env;
  setSetup( &env, setup_SDLSKK_Context );
  setTeardown( &env, teardown_SDLSKK_Context );
  
  TEST_FRAME0( test_SDLSKK_Str_insert_Char );
  TEST_FRAME0( test_SDLSKK_DictItem );
  TEST_FRAME0( test_SDLSKK_Str_insert_Str );
  TEST_FRAME0( test_SDSKK_Str_len );
  TEST_FRAME0( test_SDLSKK_Str_to_cstr );
  TEST_FRAME0( test_SDLSKK_RomKanaRuleTable );
  TEST_FRAME0( test_SDLSKK_Map );
  TEST_FRAME0( test_SDLSKK_Context__input_hiragana );
  TEST_FRAME0( test_SDLSKK_XStr );
  TEST_FRAME( test_SDLSKK_Context_get_display_Str__input_alpha_hiragana, &env);
  TEST_FRAME( test_SDLSKK_Context__input_kanji_okuri_nasi, &env );
  TEST_FRAME( test_SDLSKK_Context__input_kanji_okuri_ari, &env );
  TEST_FRAME( test_SDLSKK_Context__abbrev_mode, &env );
  TEST_FRAME( test_SDLSKK_Context_get_display_str, &env );
  TEST_FRAME( test_SDLSKK_Context__input_kigou, &env );
  TEST_FRAME( test_SDLSKK_Context__jisx0208_latin_mode, &env );
  TEST_FRAME0( test_SDLSKK_katakana_to_hiragana );
  TEST_FRAME0( test_SDLSKK_hiragana_to_katakana );
  TEST_FRAME( test_SDLSKK_Context__input_katakana, &env );
  TEST_FRAME0( test_SDLSKK_Str_hiragana_to_katakana );
  TEST_FRAME( test_SDLSKK_Context__input_cancel_key, &env );
  TEST_FRAME( test_SDLSKK_DictItem_commit, &env );
  TEST_FRAME( test_SDLSKK_Dict_save_user_jisyo, &env );
  TEST_FRAME0( test_SDLSKK_Dict_load );
}

int main( int argc, char** argv )
{
  test_run();
  putchar( '\n' );
  return 0;
}
